@@ -20,23 +20,18 @@
 package javax.tv.service.selection;
 
 public abstract class ServiceContextFactory {
-    protected ServiceContextFactory()
-    {
+    protected ServiceContextFactory() {
     }
 
-    public static ServiceContextFactory getInstance()
-    {
-        return instance;
+    public static ServiceContextFactory getInstance() {
+        return ServiceContextFactoryImpl.getInstance();
     }
 
-    public abstract ServiceContext createServiceContext()
-            throws InsufficientResourcesException, SecurityException;
+    public abstract ServiceContext createServiceContext() throws InsufficientResourcesException, SecurityException;
 
     public abstract ServiceContext[] getServiceContexts();
 
     public abstract ServiceContext getServiceContext(
             javax.tv.xlet.XletContext context) throws SecurityException,
             ServiceContextException;
-
-    private static final ServiceContextFactory instance = new ServiceContextFactoryImpl();
 }
