@@ -20,13 +20,13 @@
 package org.bluray.ti;
 
 public class CodingType {
-    protected CodingType(String name)
-    {
+    protected CodingType(String name) {
         this.name = name;
+        if (name == null)
+            throw new NullPointerException("name is null");
     }
 
-    public String toString()
-    {
+    public String toString() {
         return name;
     }
 
