@@ -20,10 +20,7 @@
 package org.bluray.ti;
 
 public class DiscInsertedEvent extends DiscStatusEvent {
-    public DiscInsertedEvent(Object source)
-    {
+    public DiscInsertedEvent(Object source) {
         super(source);
     }
-
-    private static final long serialVersionUID = -474637357700362624L;
 }
