@@ -22,8 +22,7 @@ package org.blurayx.s3d.media;
 import java.io.Serializable;
 import java.util.EventObject;
 
-public class GraphicsOffsetSequenceChangeEvent extends EventObject implements Serializable
-{
+public class GraphicsOffsetSequenceChangeEvent extends EventObject implements Serializable {
     private int sequenceId;
 
     public GraphicsOffsetSequenceChangeEvent(GraphicsOffsetSequenceControl source, int sequenceId) {
@@ -34,4 +33,4 @@ public class GraphicsOffsetSequenceChangeEvent extends EventObject implements Se
     public int getOffsetSequenceId() {
         return sequenceId;
     }
-}
\ No newline at end of file
+}
