@@ -49,6 +49,7 @@ public class BDJLoader {
             this.fontPath = null;
             this.is = is;
         }
+
         public FontCacheAction(String fontPath) {
             this.fontPath = fontPath;
             this.is = null;
@@ -242,11 +243,8 @@ public class BDJLoader {
             // initialize appProxys
             for (int i = 0; i < appTable.length; i++) {
                 if (proxys[i] == null) {
-                    proxys[i] = BDJAppProxy.newInstance(
-                                                new BDJXletContext(
-                                                                   appTable[i],
-                                                                   bdjo.getAppCaches(),
-                                                                   gui));
+                    proxys[i] = BDJAppProxy.newInstance(new BDJXletContext(appTable[i], bdjo.getAppCaches(), gui));
+
                     /* log startup class, startup parameters and jar file */
                     String[] params = appTable[i].getParams();
                     String p = "";
