@@ -26,13 +26,12 @@ import javax.media.IncompatibleSourceException;
 import javax.media.Time;
 import javax.media.protocol.DataSource;
 
-
-import org.videolan.media.content.playlist.OverallGainControlImpl;
-import org.videolan.media.content.playlist.PanningControlImpl;
+import org.videolan.media.content.control.MediaTimePositionControlImpl;
+import org.videolan.media.content.control.OverallGainControlImpl;
+import org.videolan.media.content.control.PanningControlImpl;
 
 import org.videolan.media.content.BDHandler;
 
-
 public class Handler extends BDHandler {
     public Handler() {
         controls = new Control[3];
@@ -70,10 +69,11 @@ public class Handler extends BDHandler {
         return super.doStart(at);
     }
 
-    protected ControllerErrorEvent doStop() {
-        return super.doStop();
+    /*
+    protected ControllerErrorEvent doStop(boolean eof) {
+        return super.doStop(eof);
     }
-
+    */
     /*
     protected BDLocator getLocator() {
         return locator;
