@@ -37,7 +37,8 @@ import org.videolan.TIClip;
 
 public class PiPControlImpl extends VideoControl implements PiPControl, AsynchronousPiPControl {
     protected PiPControlImpl(Handler player) {
-        super(player, 1);
+        super(1);
+        this.player = player;
     }
 
     protected StreamInfo[] getStreams() {
@@ -83,8 +84,8 @@ public class PiPControlImpl extends VideoControl implements PiPControl, Asynchro
         return false;
     }
 
-    protected void onPiPChange(int param) {
-        listeners.putCallback(new PiPStatusEvent(param > 0, this));
+    protected void onPiPStatusChange(boolean enable) {
+        listeners.putCallback(new PiPStatusEvent(enable, this));
     }
 
     public void addPiPStatusListener(PiPStatusListener listener) {
@@ -117,5 +118,6 @@ public class PiPControlImpl extends VideoControl implements PiPControl, Asynchro
         return null;
     }
 
+    private Handler player;
     private BDJListeners listeners = new BDJListeners();
 }
