@@ -117,8 +117,12 @@ static int CALLBACK EnumFontCallbackW(const ENUMLOGFONTEXW *lpelfe, const NEWTEX
             size_t len = WideCharToMultiByte(CP_UTF8, 0, wdata, -1, NULL, 0, NULL, NULL);
             if (len != 0) {
                 data->filename = (char *)malloc(len);
-                WideCharToMultiByte(CP_UTF8, 0, wdata, -1, data->filename, len, NULL, NULL);
-                break;
+                if (data->filename) {
+                    if (!WideCharToMultiByte(CP_UTF8, 0, wdata, -1, data->filename, len, NULL, NULL)) {
+                        data->filename[0] = 0;
+                    }
+                    break;
+                }
             }
         }
     }
@@ -484,7 +488,7 @@ Java_java_awt_BDFontMetrics_charsWidthN(JNIEnv * env, jobject obj, jlong ftFace,
         return 0;
     (*env)->GetCharArrayRegion(env, charArray, offset, length, chars);
     if ((*env)->ExceptionCheck(env)) {
-        free(chars);
+        X_FREE(chars);
         return 0;
     }
 
@@ -494,7 +498,7 @@ Java_java_awt_BDFontMetrics_charsWidthN(JNIEnv * env, jobject obj, jlong ftFace,
         }
     }
 
-    free(chars);
+    X_FREE(chars);
 
     return width;
 
