@@ -25,6 +25,7 @@
 #include "mpls_parse.h"
 
 #include "extdata_parse.h"
+#include "bdmv_parse.h"
 #include "uo_mask.h"
 
 #include "disc/disc.h"
@@ -37,23 +38,7 @@
 #include <stdlib.h>
 #include <string.h>
 
-#define MPLS_SIG1  ('M' << 24 | 'P' << 16 | 'L' << 8 | 'S')
-#define MPLS_SIG2A ('0' << 24 | '2' << 16 | '0' << 8 | '0')
-#define MPLS_SIG2B ('0' << 24 | '1' << 16 | '0' << 8 | '0')
-
-static void
-_human_readable_sig(char *sig, uint32_t s1, uint32_t s2)
-{
-    sig[0] = (s1 >> 24) & 0xFF;
-    sig[1] = (s1 >> 16) & 0xFF;
-    sig[2] = (s1 >>  8) & 0xFF;
-    sig[3] = (s1      ) & 0xFF;
-    sig[4] = (s2 >> 24) & 0xFF;
-    sig[5] = (s2 >> 16) & 0xFF;
-    sig[6] = (s2 >>  8) & 0xFF;
-    sig[7] = (s2      ) & 0xFF;
-    sig[8] = 0;
-}
+#define MPLS_SIG1 ('M' << 24 | 'P' << 16 | 'L' << 8 | 'S')
 
 static int
 _parse_uo(BITSTREAM *bits, BD_UO_MASK *uo)
@@ -103,26 +88,16 @@ _parse_appinfo(BITSTREAM *bits, MPLS_AI *ai)
 static int
 _parse_header(BITSTREAM *bits, MPLS_PL *pl)
 {
+    pl->type_indicator = MPLS_SIG1;
+    if (!bdmv_parse_header(bits, pl->type_indicator, &pl->type_indicator2)) {
+        return 0;
+    }
+
     if (bs_avail(bits) < 5 * 32 + 160) {
         BD_DEBUG(DBG_NAV | DBG_CRIT, "_parse_header: unexpected end of file\n");
         return 0;
     }
 
-    pl->type_indicator  = bs_read(bits, 32);
-    pl->type_indicator2 = bs_read(bits, 32);
-    if (pl->type_indicator != MPLS_SIG1 || 
-        (pl->type_indicator2 != MPLS_SIG2A && 
-         pl->type_indicator2 != MPLS_SIG2B)) {
-
-        char sig[9];
-        char expect[9];
-
-        _human_readable_sig(sig, pl->type_indicator, pl->type_indicator2);
-        _human_readable_sig(expect, MPLS_SIG1, MPLS_SIG2A);
-        BD_DEBUG(DBG_NAV | DBG_CRIT, "failed signature match, expected (%s) got (%s)\n",
-                expect, sig);
-        return 0;
-    }
     pl->list_pos = bs_read(bits, 32);
     pl->mark_pos = bs_read(bits, 32);
     pl->ext_pos  = bs_read(bits, 32);
@@ -183,6 +158,7 @@ _parse_stream(BITSTREAM *bits, MPLS_STREAM *s)
         case 0x02:
         case 0xea:
         case 0x1b:
+        case 0x24:
             s->format = bs_read(bits, 4);
             s->rate   = bs_read(bits, 4);
             break;
@@ -259,6 +235,9 @@ _parse_stn(BITSTREAM *bits, MPLS_STN *stn)
     ss = NULL;
     if (stn->num_video) {
         ss = calloc(stn->num_video, sizeof(MPLS_STREAM));
+        if (!ss) {
+            return 0;
+        }
         for (ii = 0; ii < stn->num_video; ii++) {
             if (!_parse_stream(bits, &ss[ii])) {
                 X_FREE(ss);
@@ -273,6 +252,9 @@ _parse_stn(BITSTREAM *bits, MPLS_STN *stn)
     ss = NULL;
     if (stn->num_audio) {
         ss = calloc(stn->num_audio, sizeof(MPLS_STREAM));
+        if (!ss) {
+            return 0;
+        }
         for (ii = 0; ii < stn->num_audio; ii++) {
 
             if (!_parse_stream(bits, &ss[ii])) {
@@ -288,6 +270,9 @@ _parse_stn(BITSTREAM *bits, MPLS_STN *stn)
     ss = NULL;
     if (stn->num_pg  || stn->num_pip_pg) {
         ss = calloc(stn->num_pg + stn->num_pip_pg, sizeof(MPLS_STREAM));
+        if (!ss) {
+            return 0;
+        }
         for (ii = 0; ii < (stn->num_pg + stn->num_pip_pg); ii++) {
             if (!_parse_stream(bits, &ss[ii])) {
                 X_FREE(ss);
@@ -302,6 +287,9 @@ _parse_stn(BITSTREAM *bits, MPLS_STN *stn)
     ss = NULL;
     if (stn->num_ig) {
         ss = calloc(stn->num_ig, sizeof(MPLS_STREAM));
+        if (!ss) {
+            return 0;
+        }
         for (ii = 0; ii < stn->num_ig; ii++) {
             if (!_parse_stream(bits, &ss[ii])) {
                 X_FREE(ss);
@@ -313,68 +301,79 @@ _parse_stn(BITSTREAM *bits, MPLS_STN *stn)
     stn->ig = ss;
 
     // Secondary Audio Streams
-    ss = NULL;
     if (stn->num_secondary_audio) {
         ss = calloc(stn->num_secondary_audio, sizeof(MPLS_STREAM));
+        if (!ss) {
+            return 0;
+        }
+        stn->secondary_audio = ss;
         for (ii = 0; ii < stn->num_secondary_audio; ii++) {
             if (!_parse_stream(bits, &ss[ii])) {
-                X_FREE(ss);
                 BD_DEBUG(DBG_NAV | DBG_CRIT, "error parsing secondary audio entry\n");
                 return 0;
             }
             // Read Secondary Audio Extra Attributes
-            ss->sa_num_primary_audio_ref = bs_read(bits, 8);
+            ss[ii].sa_num_primary_audio_ref = bs_read(bits, 8);
             bs_skip(bits, 8);
-            if (ss->sa_num_primary_audio_ref) {
-                ss->sa_primary_audio_ref = calloc(ss->sa_num_primary_audio_ref, sizeof(uint8_t));
-                for (jj = 0; jj < ss->sa_num_primary_audio_ref; jj++) {
-                   ss->sa_primary_audio_ref[jj] = bs_read(bits, 8);
+            if (ss[ii].sa_num_primary_audio_ref) {
+                ss[ii].sa_primary_audio_ref = calloc(ss[ii].sa_num_primary_audio_ref, sizeof(uint8_t));
+                if (!ss[ii].sa_primary_audio_ref) {
+                    return 0;
                 }
-                if (ss->sa_num_primary_audio_ref % 2) {
+                for (jj = 0; jj < ss[ii].sa_num_primary_audio_ref; jj++) {
+                   ss[ii].sa_primary_audio_ref[jj] = bs_read(bits, 8);
+                }
+                if (ss[ii].sa_num_primary_audio_ref % 2) {
                     bs_skip(bits, 8);
                 }
             }
         }
     }
-    stn->secondary_audio = ss;
 
     // Secondary Video Streams
-    ss = NULL;
     if (stn->num_secondary_video) {
         ss = calloc(stn->num_secondary_video, sizeof(MPLS_STREAM));
+        if (!ss) {
+            return 0;
+        }
+        stn->secondary_video = ss;
         for (ii = 0; ii < stn->num_secondary_video; ii++) {
             if (!_parse_stream(bits, &ss[ii])) {
-                X_FREE(ss);
                 BD_DEBUG(DBG_NAV | DBG_CRIT, "error parsing secondary video entry\n");
                 return 0;
             }
             // Read Secondary Video Extra Attributes
-            ss->sv_num_secondary_audio_ref = bs_read(bits, 8);
-           bs_skip(bits, 8);
-            if (ss->sv_num_secondary_audio_ref) {
-                ss->sv_secondary_audio_ref = calloc(ss->sv_num_secondary_audio_ref, sizeof(uint8_t));
-                for (jj = 0; jj < ss->sv_num_secondary_audio_ref; jj++) {
-                    ss->sv_secondary_audio_ref[jj] = bs_read(bits, 8);
+            ss[ii].sv_num_secondary_audio_ref = bs_read(bits, 8);
+            bs_skip(bits, 8);
+            if (ss[ii].sv_num_secondary_audio_ref) {
+                ss[ii].sv_secondary_audio_ref = calloc(ss[ii].sv_num_secondary_audio_ref, sizeof(uint8_t));
+                if (!ss[ii].sv_secondary_audio_ref) {
+                    return 0;
                 }
-                if (ss->sv_num_secondary_audio_ref % 2) {
+                for (jj = 0; jj < ss[ii].sv_num_secondary_audio_ref; jj++) {
+                    ss[ii].sv_secondary_audio_ref[jj] = bs_read(bits, 8);
+                }
+                if (ss[ii].sv_num_secondary_audio_ref % 2) {
                     bs_skip(bits, 8);
                 }
             }
-            ss->sv_num_pip_pg_ref = bs_read(bits, 8);
+            ss[ii].sv_num_pip_pg_ref = bs_read(bits, 8);
             bs_skip(bits, 8);
-            if (ss->sv_num_pip_pg_ref) {
-                ss->sv_pip_pg_ref = calloc(ss->sv_num_pip_pg_ref, sizeof(uint8_t));
-                for (jj = 0; jj < ss->sv_num_pip_pg_ref; jj++) {
-                    ss->sv_pip_pg_ref[jj] = bs_read(bits, 8);
+            if (ss[ii].sv_num_pip_pg_ref) {
+                ss[ii].sv_pip_pg_ref = calloc(ss[ii].sv_num_pip_pg_ref, sizeof(uint8_t));
+                if (!ss[ii].sv_pip_pg_ref) {
+                    return 0;
                 }
-                if (ss->sv_num_pip_pg_ref % 2) {
+                for (jj = 0; jj < ss[ii].sv_num_pip_pg_ref; jj++) {
+                    ss[ii].sv_pip_pg_ref[jj] = bs_read(bits, 8);
+                }
+                if (ss[ii].sv_num_pip_pg_ref % 2) {
                     bs_skip(bits, 8);
                 }
             }
 
         }
     }
-    stn->secondary_video = ss;
 
     if (bs_seek_byte(bits, pos + len) < 0) {
         return 0;
@@ -386,12 +385,18 @@ _parse_stn(BITSTREAM *bits, MPLS_STN *stn)
 static void
 _clean_stn(MPLS_STN *stn)
 {
+    unsigned ii;
+
     if(stn->secondary_audio) {
-        X_FREE(stn->secondary_audio->sa_primary_audio_ref);
+        for (ii = 0; ii < stn->num_secondary_audio; ii++) {
+            X_FREE(stn->secondary_audio[ii].sa_primary_audio_ref);
+        }
     }
     if(stn->secondary_video) {
-        X_FREE(stn->secondary_video->sv_secondary_audio_ref);
-        X_FREE(stn->secondary_video->sv_pip_pg_ref);
+        for (ii = 0; ii < stn->num_secondary_video; ii++) {
+            X_FREE(stn->secondary_video[ii].sv_secondary_audio_ref);
+            X_FREE(stn->secondary_video[ii].sv_pip_pg_ref);
+        }
     }
 
     X_FREE(stn->video);
@@ -418,6 +423,15 @@ _parse_playitem(BITSTREAM *bits, MPLS_PI *pi)
     len = bs_read(bits, 16);
     pos = bs_pos(bits) >> 3;
 
+    if (len < 18) {
+        BD_DEBUG(DBG_NAV | DBG_CRIT, "_parse_playitem: invalid length %d\n", len);
+        return 0;
+    }
+    if (bs_avail(bits)/8 < len) {
+        BD_DEBUG(DBG_NAV | DBG_CRIT, "_parse_playitem: unexpected EOF\n");
+        return 0;
+    }
+
     // Primary Clip identifer
     bs_read_string(bits, clip_id, 5);
 
@@ -465,6 +479,9 @@ _parse_playitem(BITSTREAM *bits, MPLS_PI *pi)
         pi->is_seamless_angle = bs_read(bits, 1);
     }
     pi->clip = calloc(pi->angle_count, sizeof(MPLS_CLIP));
+    if (!pi->clip) {
+        return 0;
+    }
     strcpy(pi->clip[0].clip_id, clip_id);
     strcpy(pi->clip[0].codec_id, codec_id);
     pi->clip[0].stc_id = stc_id;
@@ -512,6 +529,16 @@ _parse_subplayitem(BITSTREAM *bits, MPLS_SUB_PI *spi)
     len = bs_read(bits, 16);
     pos = bs_pos(bits) >> 3;
 
+    if (len < 24) {
+        BD_DEBUG(DBG_NAV | DBG_CRIT, "_parse_subplayitem: invalid length %d\n", len);
+        return 0;
+    }
+
+    if (bs_avail(bits)/8 < len) {
+        BD_DEBUG(DBG_NAV | DBG_CRIT, "_parse_subplayitem: unexpected EOF\n");
+        return 0;
+    }
+
     // Primary Clip identifer
     bs_read_string(bits, clip_id, 5);
 
@@ -545,6 +572,9 @@ _parse_subplayitem(BITSTREAM *bits, MPLS_SUB_PI *spi)
         }
     }
     spi->clip = calloc(spi->clip_count, sizeof(MPLS_CLIP));
+    if (!spi->clip) {
+        return 0;
+    }
     strcpy(spi->clip[0].clip_id, clip_id);
     strcpy(spi->clip[0].codec_id, codec_id);
     spi->clip[0].stc_id = stc_id;
@@ -596,15 +626,19 @@ _parse_subpath(BITSTREAM *bits, MPLS_SUB *sp)
     bs_skip(bits, 8);
     sp->sub_playitem_count = bs_read(bits, 8);
 
+    if (sp->sub_playitem_count) {
     spi = calloc(sp->sub_playitem_count,  sizeof(MPLS_SUB_PI));
+        if (!spi) {
+            return 0;
+        }
+    sp->sub_play_item = spi;
     for (ii = 0; ii < sp->sub_playitem_count; ii++) {
         if (!_parse_subplayitem(bits, &spi[ii])) {
-            X_FREE(spi);
             BD_DEBUG(DBG_NAV | DBG_CRIT, "error parsing sub play item\n");
             return 0;
         }
     }
-    sp->sub_play_item = spi;
+    }
 
     // Seek to end of subpath
     if (bs_seek_byte(bits, pos + len) < 0) {
@@ -639,19 +673,25 @@ _parse_playlistmark(BITSTREAM *bits, MPLS_PL *pl)
     // length field
     len = bs_read(bits, 32);
 
-    if (bs_avail(bits) < len * 8) {
-        BD_DEBUG(DBG_NAV | DBG_CRIT, "_parse_playlistmark: unexpected end of file\n");
+    if (bs_avail(bits)/8 < len) {
+        BD_DEBUG(DBG_NAV | DBG_CRIT, "_parse_playlistmark: unexpected EOF\n");
         return 0;
     }
 
     // Then get the number of marks
     pl->mark_count = bs_read(bits, 16);
 
+    if (bs_avail(bits)/(8*14) < pl->mark_count) {
+        BD_DEBUG(DBG_NAV | DBG_CRIT, "_parse_playlistmark: unexpected EOF\n");
+        return 0;
+    }
+
     plm = calloc(pl->mark_count, sizeof(MPLS_PLM));
     if (pl->mark_count && !plm) {
         BD_DEBUG(DBG_CRIT, "out of memory\n");
         return 0;
     }
+
     for (ii = 0; ii < pl->mark_count; ii++) {
         bs_skip(bits, 8); /* reserved */
         plm[ii].mark_type     = bs_read(bits, 8);
@@ -690,27 +730,35 @@ _parse_playlist(BITSTREAM *bits, MPLS_PL *pl)
     pl->list_count = bs_read(bits, 16);
     pl->sub_count = bs_read(bits, 16);
 
+    if (pl->list_count) {
     pi = calloc(pl->list_count,  sizeof(MPLS_PI));
+        if (!pi) {
+            return 0;
+        }
+    pl->play_item = pi;
     for (ii = 0; ii < pl->list_count; ii++) {
         if (!_parse_playitem(bits, &pi[ii])) {
-            X_FREE(pi);
             BD_DEBUG(DBG_NAV | DBG_CRIT, "error parsing play list item\n");
             return 0;
         }
     }
-    pl->play_item = pi;
+    }
 
+    if (pl->sub_count) {
     sub_path = calloc(pl->sub_count,  sizeof(MPLS_SUB));
+        if (!sub_path) {
+            return 0;
+        }
+    pl->sub_path = sub_path;
     for (ii = 0; ii < pl->sub_count; ii++)
     {
         if (!_parse_subpath(bits, &sub_path[ii]))
         {
-            X_FREE(sub_path);
             BD_DEBUG(DBG_NAV | DBG_CRIT, "error parsing subpath\n");
             return 0;
         }
     }
-    pl->sub_path = sub_path;
+    }
 
     return 1;
 }
