@@ -1,6 +1,6 @@
 /*
  * This file is part of libbluray
- * Copyright (C) 2014  Petri Hintukainen <phintuka@users.sourceforge.net>
+ * Copyright (C) 2014-2017  Petri Hintukainen <phintuka@users.sourceforge.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -24,6 +24,7 @@
 #include "disc.h"
 
 #include "dec.h"
+#include "properties.h"
 
 #include "util/logging.h"
 #include "util/macro.h"
@@ -31,15 +32,17 @@
 #include "util/strutl.h"
 #include "file/file.h"
 #include "file/mount.h"
+#include "file/dirs.h"
 
+#include <ctype.h>
+#include <stdio.h>
 #include <string.h>
 
-#ifdef ENABLE_UDF
 #include "udf_fs.h"
-#endif
 
 struct bd_disc {
     BD_MUTEX  ovl_mutex;     /* protect access to overlay root */
+    BD_MUTEX  properties_mutex; /* protect access to properties file */
 
     char     *disc_root;     /* disc filesystem root (if disc is mounted) */
     char     *overlay_root;  /* overlay filesystem root (if set) */
@@ -52,6 +55,9 @@ struct bd_disc {
     void        (*pf_fs_close)(void *);
 
     const char   *udf_volid;
+    char         *properties_file;  /* NULL if not yet used */
+
+    int8_t        avchd;  /* -1 - unknown. 0 - no. 1 - yes */
 };
 
 /*
@@ -92,6 +98,44 @@ static BD_DIR_H *_bdrom_open_dir(void *p, const char *dir)
     return dp;
 }
 
+/*
+ * AVCHD 8.3 filenames
+ */
+
+static char *_avchd_file_name(const char *rel_path)
+{
+    static const char map[][2][6] = {
+        { ".mpls", ".MPL" },
+        { ".clpi", ".CPI" },
+        { ".m2ts", ".MTS" },
+        { ".bdmv", ".BDM" },
+    };
+    char *avchd_path = str_dup(rel_path);
+    char *name = avchd_path ? strrchr(avchd_path, DIR_SEP_CHAR) : NULL;
+    char *dot = name ? strrchr(name, '.') : NULL;
+    size_t i;
+
+    if (dot) {
+
+        /* take up to 8 chars from file name */
+        for (i = 0; *name && name < dot && i < 9; i++, name++) {
+            *name = toupper(*name);
+        }
+
+        /* convert extension */
+        for (i = 0; i < sizeof(map) / sizeof(map[0]); i++) {
+            if (!strcmp(dot, map[i][0])) {
+                strcpy(name, map[i][1]);
+                return avchd_path;
+            }
+        }
+    }
+
+    /* failed */
+    X_FREE(avchd_path);
+    return NULL;
+}
+
 /*
  * overlay filesystem
  */
@@ -224,11 +268,14 @@ static BD_DISC *_disc_init()
     BD_DISC *p = calloc(1, sizeof(BD_DISC));
     if (p) {
         bd_mutex_init(&p->ovl_mutex);
+        bd_mutex_init(&p->properties_mutex);
 
         /* default file access functions */
         p->fs_handle          = (void*)p;
         p->pf_file_open_bdrom = _bdrom_open_path;
         p->pf_dir_open_bdrom  = _bdrom_open_dir;
+
+        p->avchd = -1;
     }
     return p;
 }
@@ -256,43 +303,43 @@ BD_DISC *disc_open(const char *device_path,
 {
     BD_DISC *p = _disc_init();
 
-    if (p) {
-        if (p_fs && p_fs->open_dir) {
-            p->fs_handle          = p_fs->fs_handle;
-            p->pf_file_open_bdrom = p_fs->open_file;
-            p->pf_dir_open_bdrom  = p_fs->open_dir;
-        }
-
-        _set_paths(p, device_path);
-
-#ifdef ENABLE_UDF
-        /* check if disc root directory can be opened. If not, treat it as device/image file. */
-        BD_DIR_H *dp_img = device_path ? dir_open(device_path) : NULL;
-        if (!dp_img) {
-            void *udf = udf_image_open(device_path, p_fs ? p_fs->fs_handle : NULL, p_fs ? p_fs->read_blocks : NULL);
-            if (!udf) {
-                BD_DEBUG(DBG_FILE | DBG_CRIT, "failed opening UDF image %s\n", device_path);
-            } else {
-                p->fs_handle          = udf;
-                p->pf_fs_close        = udf_image_close;
-                p->pf_file_open_bdrom = udf_file_open;
-                p->pf_dir_open_bdrom  = udf_dir_open;
-
-                p->udf_volid = udf_volume_id(udf);
-
-                /* root not accessible with stdio */
-                X_FREE(p->disc_root);
-            }
-        } else {
-            dir_close(dp_img);
-            BD_DEBUG(DBG_FILE, "%s does not seem to be image file or device node\n", device_path);
-        }
-#endif
-
-        struct dec_dev dev = { p->fs_handle, p->pf_file_open_bdrom, p, (file_openFp)disc_open_path, p->disc_root, device_path };
-        p->dec = dec_init(&dev, enc_info, keyfile_path, regs, psr_read, psr_write);
+    if (!p) {
+        return NULL;
     }
 
+    if (p_fs && p_fs->open_dir) {
+        p->fs_handle          = p_fs->fs_handle;
+        p->pf_file_open_bdrom = p_fs->open_file;
+        p->pf_dir_open_bdrom  = p_fs->open_dir;
+    }
+
+    _set_paths(p, device_path);
+
+    /* check if disc root directory can be opened. If not, treat it as device/image file. */
+    BD_DIR_H *dp_img = device_path ? dir_open(device_path) : NULL;
+    if (!dp_img) {
+        void *udf = udf_image_open(device_path, p_fs ? p_fs->fs_handle : NULL, p_fs ? p_fs->read_blocks : NULL);
+        if (!udf) {
+            BD_DEBUG(DBG_FILE | DBG_CRIT, "failed opening UDF image %s\n", device_path);
+        } else {
+            p->fs_handle          = udf;
+            p->pf_fs_close        = udf_image_close;
+            p->pf_file_open_bdrom = udf_file_open;
+            p->pf_dir_open_bdrom  = udf_dir_open;
+
+            p->udf_volid = udf_volume_id(udf);
+
+            /* root not accessible with stdio */
+            X_FREE(p->disc_root);
+        }
+    } else {
+        dir_close(dp_img);
+        BD_DEBUG(DBG_FILE, "%s does not seem to be image file or device node\n", device_path);
+    }
+
+    struct dec_dev dev = { p->fs_handle, p->pf_file_open_bdrom, p, (file_openFp)disc_open_path, p->disc_root, device_path };
+    p->dec = dec_init(&dev, enc_info, keyfile_path, regs, psr_read, psr_write);
+
     return p;
 }
 
@@ -308,8 +355,10 @@ void disc_close(BD_DISC **pp)
         }
 
         bd_mutex_destroy(&p->ovl_mutex);
+        bd_mutex_destroy(&p->properties_mutex);
 
         X_FREE(p->disc_root);
+        X_FREE(p->properties_file);
         X_FREE(*pp);
     }
 }
@@ -341,6 +390,18 @@ BD_FILE_H *disc_open_path(BD_DISC *p, const char *rel_path)
 {
     BD_FILE_H *fp;
 
+    if (p->avchd > 0) {
+        char *avchd_path = _avchd_file_name(rel_path);
+        if (avchd_path) {
+            BD_DEBUG(DBG_FILE, "AVCHD: %s -> %s\n", rel_path, avchd_path);
+            fp = p->pf_file_open_bdrom(p->fs_handle, avchd_path);
+            X_FREE(avchd_path);
+            if (fp) {
+                return fp;
+            }
+        }
+    }
+
     /* search file from overlay */
     fp = _overlay_open_path(p, rel_path);
 
@@ -349,7 +410,19 @@ BD_FILE_H *disc_open_path(BD_DISC *p, const char *rel_path)
         fp = p->pf_file_open_bdrom(p->fs_handle, rel_path);
 
         if (!fp) {
-            BD_DEBUG(DBG_FILE | DBG_CRIT, "error opening file %s\n", rel_path);
+
+            /* AVCHD short filenames detection */
+            if (p->avchd < 0 && !strcmp(rel_path, "BDMV" DIR_SEP "index.bdmv")) {
+                fp = p->pf_file_open_bdrom(p->fs_handle, "BDMV" DIR_SEP "INDEX.BDM");
+                if (fp) {
+                    BD_DEBUG(DBG_FILE | DBG_CRIT, "detected AVCHD 8.3 filenames\n");
+                }
+                p->avchd = !!fp;
+            }
+
+            if (!fp) {
+                BD_DEBUG(DBG_FILE | DBG_CRIT, "error opening file %s\n", rel_path);
+            }
         }
     }
 
@@ -453,9 +526,20 @@ int disc_cache_bdrom_file(BD_DISC *p, const char *rel_path, const char *cache_pa
     BD_FILE_H *fp_in;
     BD_FILE_H *fp_out;
     int64_t    got;
+    size_t     size;
 
-    if (rel_path[strlen(rel_path) - 1] == '/') {
-        file_mkdirs(cache_path);
+    if (!cache_path || !cache_path[0]) {
+        return -1;
+    }
+
+    /* make sure cache directory exists */
+    if (file_mkdirs(cache_path) < 0) {
+        return -1;
+    }
+
+    /* plain directory ? */
+    size = strlen(rel_path);
+    if (rel_path[size - 1] == '/' || rel_path[size - 1] == '\\') {
         return 0;
     }
 
@@ -466,9 +550,6 @@ int disc_cache_bdrom_file(BD_DISC *p, const char *rel_path, const char *cache_pa
         return -1;
     }
 
-    /* make sure path exists */
-    file_mkdirs(cache_path);
-
     /* output file in local filesystem */
     fp_out = file_open(cache_path, "wb");
     if (!fp_out) {
@@ -498,6 +579,84 @@ int disc_cache_bdrom_file(BD_DISC *p, const char *rel_path, const char *cache_pa
     return 0;
 }
 
+/*
+ * persistent properties storage
+ */
+
+static char *_properties_file(BD_DISC *p)
+{
+    const uint8_t *disc_id = NULL;
+    uint8_t  pseudo_id[20];
+    char     id_type, id_str[41];
+    char    *cache_home;
+    char    *properties_file;
+
+    cache_home = file_get_cache_home();
+    if (!cache_home) {
+        return NULL;
+    }
+
+    /* get disc ID */
+    if (p->dec) {
+        id_type = 'A';
+        disc_id = dec_disc_id(p->dec);
+    }
+    if (!disc_id) {
+        id_type = 'P';
+        disc_pseudo_id(p, pseudo_id);
+        disc_id = pseudo_id;
+    }
+
+    properties_file = str_printf("%s" DIR_SEP "bluray" DIR_SEP "properties" DIR_SEP "%c%s",
+                                 cache_home, id_type,
+                                 str_print_hex(id_str, disc_id, 20));
+
+    X_FREE(cache_home);
+
+    return properties_file;
+}
+
+static int _ensure_properties_file(BD_DISC *p)
+{
+    bd_mutex_lock(&p->properties_mutex);
+    if (!p->properties_file) {
+        p->properties_file = _properties_file(p);
+    }
+    bd_mutex_unlock(&p->properties_mutex);
+
+    return p->properties_file ? 0 : -1;
+}
+
+int disc_property_put(BD_DISC *p, const char *property, const char *val)
+{
+    int result;
+
+    if (_ensure_properties_file(p) < 0) {
+        return -1;
+    }
+
+    bd_mutex_lock(&p->properties_mutex);
+    result = properties_put(p->properties_file, property, val);
+    bd_mutex_unlock(&p->properties_mutex);
+
+    return result;
+}
+
+char *disc_property_get(BD_DISC *p, const char *property)
+{
+    char *result;
+
+    if (_ensure_properties_file(p) < 0) {
+        return NULL;
+    }
+
+    bd_mutex_lock(&p->properties_mutex);
+    result = properties_get(p->properties_file, property);
+    bd_mutex_unlock(&p->properties_mutex);
+
+    return result;
+}
+
 /*
  * streams
  */
@@ -529,7 +688,7 @@ const uint8_t *disc_get_data(BD_DISC *disc, int type)
 
 void disc_event(BD_DISC *disc, uint32_t event, uint32_t param)
 {
-    if (disc->dec) {
+    if (disc && disc->dec) {
         switch (event) {
             case DISC_EVENT_START:
                 dec_start(disc->dec, param);
